# Generated by Django 5.2.8 on 2025-11-18 19:08

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_telegram_app', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='telegramsettings',
            name='user',
        ),
    ]
